@@ -37,7 +37,12 @@ from .NodeMakingHelpers import (
     wrapExpressionWithNodeSideEffects,
     wrapExpressionWithSideEffects,
 )
-from .shapes.BuiltinTypeShapes import tshape_dict, tshape_str, tshape_unicode
+from .shapes.BuiltinTypeShapes import (
+    tshape_bytes,
+    tshape_dict,
+    tshape_str,
+    tshape_unicode,
+)
 from .shapes.StandardShapes import tshape_unknown
 
 
@@ -845,6 +850,10 @@ class ExpressionBase(NodeBase):
         def hasShapeStrOrUnicodeExact(self):
             return self.getTypeShape() is tshape_str
 
+    def hasShapeBytesExact(self):
+        """Does an expression have exactly a bytes shape."""
+        return self.getTypeShape() is tshape_bytes
+
 
 class ExpressionNoSideEffectsMixin(object):
     __slots__ = ()
@@ -1034,9 +1043,8 @@ Compile time constant bytes value pre-computed.""",
         # If it raises, or the attribute itself is a compile time constant,
         # then do execute it.
         if not self.computed_attribute or isCompileTimeConstantValue(
-            getattr(value, attribute_name)
+            getattr(value, attribute_name, None)
         ):
-
             return trace_collection.getCompileTimeComputationResult(
                 node=lookup_node,
                 computation=lambda: getattr(value, attribute_name),
