@@ -30,10 +30,19 @@ import sys
 
 from nuitka.__past__ import long, unicode, xrange
 from nuitka.codegen.Reports import onMissingTrust
-from nuitka.importing.Importing import isPackageDir, locateModule
+from nuitka.importing.Importing import (
+    getModuleNameAndKindFromFilename,
+    isPackageDir,
+    locateModule,
+)
 from nuitka.importing.ImportResolving import resolveModuleName
+from nuitka.importing.Recursion import decideRecursion
 from nuitka.importing.StandardLibrary import isStandardLibraryPath
-from nuitka.Options import isStandaloneMode, shallWarnUnusualCode
+from nuitka.Options import (
+    isStandaloneMode,
+    shallMakeModule,
+    shallWarnUnusualCode,
+)
 from nuitka.PythonVersions import (
     getFutureModuleKeys,
     getImportlibSubPackages,
@@ -55,9 +64,21 @@ from .ExpressionBases import (
     ExpressionChildHavingBase,
     ExpressionChildrenHavingBase,
 )
+from .ImportHardNodes import (
+    ExpressionImportHardBase,
+    ExpressionImportModuleNameHardExists,
+    ExpressionImportModuleNameHardMaybeExists,
+)
 from .LocalsScopes import GlobalsDictHandle
 from .NodeBases import StatementChildHavingBase
 from .NodeMakingHelpers import makeRaiseExceptionReplacementExpression
+from .PackageResourceNodes import (
+    ExpressionImportlibResourcesReadBinaryRef,
+    ExpressionImportlibResourcesReadTextRef,
+    ExpressionPkglibGetDataRef,
+    ExpressionPkgResourcesResourceStreamRef,
+    ExpressionPkgResourcesResourceStringRef,
+)
 from .shapes.BuiltinTypeShapes import tshape_module, tshape_module_builtin
 
 # These module are supported in code generation to be imported the hard way.
@@ -68,21 +89,55 @@ hard_modules = frozenset(
         "types",
         "typing",
         "__future__",
-        "site",
         "importlib",
+        "importlib.resources",
+        "importlib.metadata",
         "_frozen_importlib",
         "_frozen_importlib_external",
         "pkgutil",
         "functools",
+        "sysconfig",
+        # "cStringIO",
+        "io",
     )
 )
 
+# Lets put here, hard modules that are kind of backports only.
+hard_modules_non_stdlib = frozenset(
+    (
+        "site",
+        "pkg_resources",
+        "importlib_metadata",
+    )
+)
+
+hard_modules = hard_modules | hard_modules_non_stdlib
+
 hard_modules_version = {
-    "typing": 0x350,
-    "_frozen_importlib": 0x300,
-    "_frozen_importlib_external": 0x350,
+    "cStringIO": (None, 0x300),
+    "typing": (0x350, None),
+    "_frozen_importlib": (0x300, None),
+    "_frozen_importlib_external": (0x350, None),
+    "importlib.resources": (0x370, None),
+    "importlib.metadata": (0x390, None),
 }
 
+
+def isHardModule(module_name):
+    if module_name not in hard_modules:
+        return False
+
+    min_version, max_version = hard_modules_version.get(module_name, (None, None))
+
+    if min_version is not None and python_version < min_version:
+        return False
+
+    if max_version is not None and python_version >= max_version:
+        return False
+
+    return True
+
+
 trust_undefined = 0
 trust_constant = 1
 trust_exist = 2
@@ -91,7 +146,6 @@ trust_importable = 3
 trust_node = 4
 trust_may_exist = 5
 trust_not_exist = 6
-
 trust_node_factory = {}
 
 module_importlib_trust = dict(
@@ -138,19 +192,101 @@ hard_modules_trust = {
     "types": {},
     "typing": module_typing_trust,
     "__future__": dict((key, trust_future) for key in getFutureModuleKeys()),
-    "site": {},
     "importlib": module_importlib_trust,
+    "importlib.metadata": {},
+    "importlib_metadata": {},
     "_frozen_importlib": {},
     "_frozen_importlib_external": {},
-    "pkgutil": {"get_data": trust_exist},
+    "pkgutil": {"get_data": trust_node},
     "functools": {"partial": trust_exist},
+    "sysconfig": {},
+    "io": {"BytesIO": trust_exist},
+    # "cStringIO": {"StringIO": trust_exist},
+    "pkg_resources": {"resource_string": trust_node, "resource_stream": trust_node},
+    "importlib.resources": {"read_binary": trust_node, "read_text": trust_node},
+    "site": {},
 }
 
 
+trust_node_factory[("pkgutil", "get_data")] = ExpressionPkglibGetDataRef
+trust_node_factory[
+    ("pkg_resources", "resource_string")
+] = ExpressionPkgResourcesResourceStringRef
+trust_node_factory[
+    ("pkg_resources", "resource_stream")
+] = ExpressionPkgResourcesResourceStreamRef
+trust_node_factory[
+    ("importlib.resources", "read_binary")
+] = ExpressionImportlibResourcesReadBinaryRef
+trust_node_factory[
+    ("importlib.resources", "read_text")
+] = ExpressionImportlibResourcesReadTextRef
+
+
+def _checkHardModules():
+    for module_name in hard_modules:
+        assert module_name in hard_modules_trust, module_name
+
+    for module_name, trust in hard_modules_trust.items():
+        assert module_name in hard_modules, module_name
+
+        for attribute_name, trust_value in trust.items():
+            if trust_value is trust_node_factory:
+                assert (module_name, attribute_name) in trust_node_factory, module_name
+
+
+_checkHardModules()
+
+
+def makeExpressionImportModuleNameHard(
+    module_name, import_name, module_guaranteed, source_ref
+):
+    if hard_modules_trust[module_name].get(import_name) is None:
+        return ExpressionImportModuleNameHardMaybeExists(
+            module_name=module_name,
+            import_name=import_name,
+            module_guaranteed=module_guaranteed,
+            source_ref=source_ref,
+        )
+    else:
+        return ExpressionImportModuleNameHardExists(
+            module_name=module_name,
+            import_name=import_name,
+            module_guaranteed=module_guaranteed,
+            source_ref=source_ref,
+        )
+
+
 def isHardModuleWithoutSideEffect(module_name):
     return module_name in hard_modules and module_name != "site"
 
 
+class ExpressionImportAllowanceMixin(object):
+    # Mixins are not allow to specify slots, pylint: disable=assigning-non-slot
+    __slots__ = ()
+
+    def __init__(self):
+        if self.finding == "not-found":
+            self.allowed = False
+        elif self.finding == "built-in":
+            self.allowed = True
+        else:
+            _module_name, module_kind = getModuleNameAndKindFromFilename(
+                self.module_filename
+            )
+
+            self.allowed, _reason = decideRecursion(
+                module_filename=self.module_filename,
+                module_name=self.module_name,
+                module_kind=module_kind,
+            )
+
+            # In case of hard imports, that are not forbidden explicitly, allow their use
+            # anyway.
+            if self.allowed is None and self.isExpressionImportModuleHard():
+                self.allowed = True
+
+
 class ExpressionImportModuleFixed(ExpressionBase):
     """Hard coded import names, that we know to exist."
 
@@ -165,12 +301,13 @@ class ExpressionImportModuleFixed(ExpressionBase):
         "found_module_name",
         "found_module_filename",
         "finding",
+        "allowance",
     )
 
     def __init__(self, module_name, source_ref):
         ExpressionBase.__init__(self, source_ref=source_ref)
 
-        self.module_name = resolveModuleName(module_name)
+        self.module_name = ModuleName(module_name)
 
         self.finding = None
 
@@ -242,7 +379,8 @@ class ExpressionImportModuleFixed(ExpressionBase):
         return self, None, None
 
     def computeExpressionImportName(self, import_node, import_name, trace_collection):
-        # TODO: For include modules, something might be possible here.
+        # TODO: For include modules, something might be possible here, consider self.allowance
+        # when that is implemented.
         return self.computeExpressionAttribute(
             lookup_node=import_node,
             attribute_name=import_name,
@@ -250,34 +388,9 @@ class ExpressionImportModuleFixed(ExpressionBase):
         )
 
 
-class ExpressionImportHardBase(ExpressionBase):
-    # Base classes can be abstract, pylint: disable=abstract-method
-    #
-    __slots__ = ("module_name", "finding", "module_filename")
-
-    def __init__(self, module_name, source_ref):
-        ExpressionBase.__init__(self, source_ref=source_ref)
-
-        self.module_name = ModuleName(module_name)
-
-        self.finding = None
-        self.module_filename = None
-
-        _module_name, self.module_filename, self.finding = locateModule(
-            module_name=self.module_name,
-            parent_package=None,
-            level=0,
-        )
-
-        # Expect to find them and to match the name of course.
-        assert self.finding != "not-found", self.module_name
-        assert _module_name == self.module_name
-
-    def getUsedModule(self):
-        return self.module_name, self.module_filename, self.finding
-
-
-class ExpressionImportModuleHard(ExpressionImportHardBase):
+class ExpressionImportModuleHard(
+    ExpressionImportAllowanceMixin, ExpressionImportHardBase
+):
     """Hard coded import names, e.g. of "__future__"
 
     These are directly created for some Python mechanics, but also due to
@@ -286,32 +399,49 @@ class ExpressionImportModuleHard(ExpressionImportHardBase):
 
     kind = "EXPRESSION_IMPORT_MODULE_HARD"
 
-    __slots__ = ("module",)
+    __slots__ = ("module", "allowed", "guaranteed", "value_name", "is_package")
 
-    def __init__(self, module_name, source_ref):
+    def __init__(self, module_name, value_name, source_ref):
         ExpressionImportHardBase.__init__(
             self, module_name=module_name, source_ref=source_ref
         )
 
-        if isHardModuleWithoutSideEffect(self.module_name):
-            self.module = __import__(self.module_name)
+        ExpressionImportAllowanceMixin.__init__(self)
+
+        self.value_name = value_name
+
+        if self.finding != "not-found" and isHardModuleWithoutSideEffect(
+            self.module_name
+        ):
+            __import__(self.module_name)
+            self.module = sys.modules[self.value_name]
+
+            self.is_package = hasattr(self.module, "__path__")
         else:
             self.module = None
+            self.is_package = None
+
+        self.guaranteed = self.allowed and (
+            not shallMakeModule() or self.module_name not in hard_modules_non_stdlib
+        )
 
     def finalize(self):
         del self.parent
 
     def getDetails(self):
-        return {"module_name": self.module_name}
+        return {"module_name": self.module_name, "value_name": self.value_name}
 
     def getModuleName(self):
         return self.module_name
 
+    def getValueName(self):
+        return self.value_name
+
     def mayHaveSideEffects(self):
-        return self.module is None
+        return self.module is None or not self.guaranteed
 
     def mayRaiseException(self, exception_type):
-        return self.mayHaveSideEffects()
+        return not self.allowed or self.mayHaveSideEffects()
 
     def getTypeShape(self):
         if self.module_name in sys.builtin_module_names:
@@ -320,19 +450,17 @@ class ExpressionImportModuleHard(ExpressionImportHardBase):
             return tshape_module
 
     def computeExpressionRaw(self, trace_collection):
-        if self.finding is None:
-            self._attemptFollow()
-
         if self.mayRaiseException(BaseException):
             trace_collection.onExceptionRaiseExit(BaseException)
 
         return self, None, None
 
     def computeExpressionImportName(self, import_node, import_name, trace_collection):
-        return self.computeExpressionAttribute(
+        return self._computeExpressionAttribute(
             lookup_node=import_node,
             attribute_name=import_name,
             trace_collection=trace_collection,
+            is_import=True,
         )
 
     @staticmethod
@@ -350,12 +478,49 @@ class ExpressionImportModuleHard(ExpressionImportHardBase):
                 module.__file__ if hasattr(module, "__file__") else "unknown location",
             )
 
-    def computeExpressionAttribute(self, lookup_node, attribute_name, trace_collection):
-        # By default, an attribute lookup may change everything about the lookup
-        # source.
+    def _makeRaiseExceptionReplacementExpression(
+        self, lookup_node, attribute_name, is_import
+    ):
+        if is_import:
+            return makeRaiseExceptionReplacementExpression(
+                expression=lookup_node,
+                exception_type="ImportError",
+                exception_value=self._getImportNameErrorString(
+                    self.module, self.value_name, attribute_name
+                ),
+            )
+        else:
+            return makeRaiseExceptionReplacementExpression(
+                expression=lookup_node,
+                exception_type="AttributeError",
+                # TODO: Not the right error message
+                exception_value=self._getImportNameErrorString(
+                    self.module, self.value_name, attribute_name
+                ),
+            )
 
-        if self.module is not None:
-            trust = hard_modules_trust[self.module_name].get(
+    def _computeExpressionAttribute(
+        self, lookup_node, attribute_name, trace_collection, is_import
+    ):
+        # Return driven handling of many cases, pylint: disable=too-many-return-statements
+
+        if self.module is not None and self.allowed:
+            full_name = self.value_name.getChildNamed(attribute_name)
+
+            if full_name in hard_modules:
+                new_node = makeExpressionImportModuleFixed(
+                    module_name=full_name,
+                    source_ref=lookup_node.source_ref,
+                )
+
+                return (
+                    new_node,
+                    "new_expression",
+                    "Hard module '%s' submodule %r pre-computed."
+                    % (self.value_name, attribute_name),
+                )
+
+            trust = hard_modules_trust[self.value_name].get(
                 attribute_name, trust_undefined
             )
 
@@ -366,35 +531,57 @@ class ExpressionImportModuleHard(ExpressionImportHardBase):
                 trace_collection.onExceptionRaiseExit(BaseException)
             elif trust is trust_may_exist:
                 trace_collection.onExceptionRaiseExit(BaseException)
-            elif (
-                not hasattr(self.module, attribute_name)
-                and trust is not trust_undefined
+            elif trust is not trust_undefined and not hasattr(
+                self.module, attribute_name
             ):
                 # TODO: Unify with below branches.
                 trace_collection.onExceptionRaiseExit(ImportError)
 
-                new_node = makeRaiseExceptionReplacementExpression(
-                    expression=lookup_node,
-                    exception_type="AttributeError",
-                    exception_value=self._getImportNameErrorString(
-                        self.module, self.module_name, attribute_name
-                    ),
+                new_node = self._makeRaiseExceptionReplacementExpression(
+                    lookup_node=lookup_node,
+                    attribute_name=attribute_name,
+                    is_import=is_import,
                 )
 
                 return (
                     new_node,
                     "new_raise",
-                    "Hard module %r attribute missing %r pre-computed."
-                    % (self.module_name, attribute_name),
+                    "Hard module %s attribute missing %r pre-computed."
+                    % (self.value_name, attribute_name),
                 )
             else:
                 if trust is trust_undefined:
+                    # TODO: Should add this, such that these imports are
+                    # properly resolved: pylint: disable=condition-evals-to-constant
+
+                    if self.is_package and False:
+                        full_name = self.value_name.getChildNamed(attribute_name)
+
+                        _sub_module_name, _sub_module_filename, finding = locateModule(
+                            module_name=full_name,
+                            parent_package=None,
+                            level=0,
+                        )
+
+                        if finding != "not-found":
+                            result = makeExpressionImportModuleFixed(
+                                module_name=full_name,
+                                source_ref=lookup_node.getSourceReference(),
+                            )
+
+                            return (
+                                result,
+                                "new_expression",
+                                "Attribute lookup %s of hard module %r becomes hard module name import."
+                                % (self.value_name, attribute_name),
+                            )
+
                     trace_collection.onExceptionRaiseExit(ImportError)
 
                     onMissingTrust(
-                        "Hard module %r attribute %r missing trust config for existing value.",
+                        "Hard module %s attribute %r missing trust config for existing value.",
                         lookup_node.getSourceReference(),
-                        self.module_name,
+                        self.value_name,
                         attribute_name,
                     )
                 elif trust is trust_constant:
@@ -408,32 +595,33 @@ class ExpressionImportModuleHard(ExpressionImportHardBase):
                             user_provided=True,
                         ),
                         "new_constant",
-                        "Hard module '%s' imported %r pre-computed to constant value."
-                        % (self.module_name.asString(), attribute_name),
+                        "Hard module %s imported %r pre-computed to constant value."
+                        % (self.value_name, attribute_name),
                     )
                 elif trust is trust_node:
-                    result = trust_node_factory[self.module_name, attribute_name](
+                    result = trust_node_factory[self.value_name, attribute_name](
                         source_ref=lookup_node.source_ref
                     )
 
                     return (
                         result,
                         "new_expression",
-                        "Attribute lookup %r of hard module %r becomes node %r."
-                        % (self.module_name.asString(), attribute_name, result.kind),
+                        "Attribute lookup %s of hard module %r becomes node %r."
+                        % (self.value_name, attribute_name, result.kind),
                     )
                 else:
-                    result = ExpressionImportModuleNameHard(
-                        module_name=self.module_name,
+                    result = makeExpressionImportModuleNameHard(
+                        module_name=self.value_name,
                         import_name=attribute_name,
+                        module_guaranteed=self.guaranteed,
                         source_ref=lookup_node.getSourceReference(),
                     )
 
                     return (
                         result,
                         "new_expression",
-                        "Attribute lookup %r of hard module %r becomes hard module name import."
-                        % (self.module_name, attribute_name),
+                        "Attribute lookup %s of hard module %r becomes hard module name import."
+                        % (self.value_name, attribute_name),
                     )
 
         else:
@@ -443,53 +631,16 @@ class ExpressionImportModuleHard(ExpressionImportHardBase):
 
         return lookup_node, None, None
 
-    def hasShapeTrustedAttributes(self):
-        return True
-
-
-class ExpressionImportModuleNameHard(ExpressionImportHardBase):
-    """Hard coded import names, e.g. of "os.path.dirname"
-
-    These are directly created for some Python mechanics.
-    """
-
-    kind = "EXPRESSION_IMPORT_MODULE_NAME_HARD"
-
-    __slots__ = ("import_name", "trust", "finding", "module_filename")
-
-    def __init__(self, module_name, import_name, source_ref):
-        ExpressionImportHardBase.__init__(
-            self, module_name=module_name, source_ref=source_ref
+    def computeExpressionAttribute(self, lookup_node, attribute_name, trace_collection):
+        return self._computeExpressionAttribute(
+            lookup_node=lookup_node,
+            attribute_name=attribute_name,
+            trace_collection=trace_collection,
+            is_import=False,
         )
 
-        self.import_name = import_name
-
-        self.trust = hard_modules_trust[self.module_name].get(self.import_name)
-
-    def finalize(self):
-        del self.parent
-
-    def getDetails(self):
-        return {"module_name": self.module_name, "import_name": self.import_name}
-
-    def getModuleName(self):
-        return self.module_name
-
-    def getImportName(self):
-        return self.import_name
-
-    def computeExpressionRaw(self, trace_collection):
-        # As good as it gets, will exist, otherwise we do not get created.
-        if self.mayHaveSideEffects():
-            trace_collection.onExceptionRaiseExit(AttributeError)
-
-        return self, None, None
-
-    def mayHaveSideEffects(self):
-        return self.trust is None
-
-    def mayRaiseException(self, exception_type):
-        return self.trust is None
+    def hasShapeTrustedAttributes(self):
+        return True
 
 
 importlib_import_module_spec = BuiltinParameterSpec(
@@ -497,14 +648,15 @@ importlib_import_module_spec = BuiltinParameterSpec(
 )
 
 
-class ExpressionImportlibImportModuleRef(ExpressionImportModuleNameHard):
+class ExpressionImportlibImportModuleRef(ExpressionImportModuleNameHardExists):
     kind = "EXPRESSION_IMPORTLIB_IMPORT_MODULE_REF"
 
     def __init__(self, source_ref):
-        ExpressionImportModuleNameHard.__init__(
+        ExpressionImportModuleNameHardExists.__init__(
             self,
             module_name="importlib",
             import_name="import_module",
+            module_guaranteed=True,
             source_ref=source_ref,
         )
 
@@ -768,7 +920,8 @@ class ExpressionBuiltinImport(ExpressionChildrenHavingBase):
         return self.used_modules
 
     def computeExpression(self, trace_collection):
-        # Attempt to recurse if not already done.
+        # Attempt to recurse if not already done, many cases to consider
+        # pylint: disable=too-many-branches
         if self.follow_attempted:
             if self.finding == "not-found":
                 # Importing and not finding, may raise an exception obviously.
@@ -780,6 +933,11 @@ class ExpressionBuiltinImport(ExpressionChildrenHavingBase):
             # We stay here.
             return self, None, None
 
+        # Importing may raise an exception obviously, unless we know it will
+        # not.
+        if self.finding != "built-in":
+            trace_collection.onExceptionRaiseExit(BaseException)
+
         module_name = self.subnode_name
 
         if module_name.isCompileTimeConstant():
@@ -792,10 +950,25 @@ class ExpressionBuiltinImport(ExpressionChildrenHavingBase):
             if type(imported_module_name) in (str, unicode):
                 imported_module_name = resolveModuleName(imported_module_name)
 
-                if self.finding == "absolute" and imported_module_name in hard_modules:
-                    if isStandardLibraryPath(module_filename):
+                if self.finding == "absolute" and isHardModule(imported_module_name):
+                    if (
+                        imported_module_name in hard_modules_non_stdlib
+                        or isStandardLibraryPath(module_filename)
+                    ):
+                        from_list_truth = (
+                            self.subnode_fromlist is not None
+                            and self.subnode_fromlist.getTruthValue()
+                        )
+
+                        if from_list_truth is True:
+                            value_name = imported_module_name
+                        else:
+                            value_name = imported_module_name.getTopLevelPackageName()
+
                         result = ExpressionImportModuleHard(
-                            module_name=imported_module_name, source_ref=self.source_ref
+                            module_name=imported_module_name,
+                            value_name=value_name,
+                            source_ref=self.source_ref,
                         )
 
                         return (
@@ -815,9 +988,11 @@ class ExpressionBuiltinImport(ExpressionChildrenHavingBase):
                         )
 
                 if self.finding == "built-in":
-                    if imported_module_name in hard_modules:
+                    if isHardModule(imported_module_name):
                         result = ExpressionImportModuleHard(
-                            module_name=imported_module_name, source_ref=self.source_ref
+                            module_name=imported_module_name,
+                            value_name=imported_module_name.getTopLevelPackageName(),
+                            source_ref=self.source_ref,
                         )
 
                         return (
@@ -851,11 +1026,6 @@ class ExpressionBuiltinImport(ExpressionChildrenHavingBase):
 
                 return new_node, change_tags, message
 
-        # Importing may raise an exception obviously, unless we know it will
-        # not.
-        if self.finding != "built-in":
-            trace_collection.onExceptionRaiseExit(BaseException)
-
         # TODO: May return a module or module variable reference of some sort in
         # the future with embedded modules.
         return self, None, None
@@ -963,9 +1133,13 @@ class ExpressionImportName(ExpressionChildHavingBase):
 
 
 def makeExpressionImportModuleFixed(module_name, source_ref):
-    if module_name in hard_modules:
+    module_name = resolveModuleName(module_name)
+
+    if isHardModule(module_name):
         return ExpressionImportModuleHard(
-            module_name=module_name, source_ref=source_ref
+            module_name=module_name,
+            value_name=module_name.getTopLevelPackageName(),
+            source_ref=source_ref,
         )
     else:
         return ExpressionImportModuleFixed(
