@@ -27,8 +27,6 @@ quickly, in others they will present boundaries for optimization.
 
 """
 
-import os
-
 from nuitka import Options
 
 from .ConstantRefNodes import makeConstantRefNode
@@ -106,9 +104,9 @@ class ExpressionModuleAttributeNameRef(ExpressionModuleAttributeBase):
     def computeExpressionRaw(self, trace_collection):
         # For binaries, we can know it definite, but not for modules.
 
-        if not Options.shallMakeModule():
+        if Options.getModuleNameMode() != "runtime":
             result = makeConstantRefNode(
-                constant=self.variable.getModule().getFullName().asString(),
+                constant=self.variable.getModule().getRuntimeNameValue(),
                 source_ref=self.source_ref,
             )
 
@@ -130,25 +128,9 @@ class ExpressionModuleAttributePackageRef(ExpressionModuleAttributeBase):
     def computeExpressionRaw(self, trace_collection):
         # For binaries, we can know it definite, but not for modules.
 
-        if not Options.shallMakeModule():
+        if Options.getModuleNameMode() != "runtime":
             provider = self.variable.getModule()
-
-            if provider.isCompiledPythonPackage():
-                value = provider.getFullName().asString()
-            else:
-                value = provider.getFullName().getPackageName()
-
-                if value is not None:
-                    value = value.asString()
-                else:
-                    if (
-                        provider.getFullName() == "__main__"
-                        and os.path.basename(provider.getCompileTimeFilename())
-                        == "__main__.py"
-                    ):
-                        value = ""
-                    else:
-                        value = None
+            value = provider.getRuntimePackageValue()
 
             result = makeConstantRefNode(constant=value, source_ref=self.source_ref)
 
