@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -34,10 +34,14 @@ allowed_tags = (
     "new_statements",
     # New expression added.
     "new_expression",
+    # Loop analysis is incomplete, or only just now completed.
+    "loop_analysis",
     # TODO: A bit unclear what this it, potentially a changed variable.
     "var_usage",
     # Detected module variable to be read only.
     "read_only_mvar",
+    # Trusting module variables in functions.
+    "trusted_module_variables",
     # New built-in reference detected.
     "new_builtin_ref",
     # New built-in call detected.
