@@ -24,21 +24,13 @@ own dependencies.
 import os
 
 from nuitka.containers.oset import OrderedSet
-from nuitka.freezer.IncludedDataFiles import (
-    makeIncludedDataFile,
-    makeIncludedGeneratedDataFile,
-)
-from nuitka.freezer.IncludedEntryPoints import (
-    makeDllEntryPoint,
-    makeExeEntryPoint,
-)
-from nuitka.Options import isStandaloneMode
+from nuitka.freezer.IncludedEntryPoints import makeExeEntryPoint
+from nuitka.Options import isExperimental, isStandaloneMode
 from nuitka.plugins.PluginBase import NuitkaPluginBase
 from nuitka.plugins.Plugins import getActiveQtPlugin
 from nuitka.PythonVersions import python_version
 from nuitka.utils.FileOperations import getFileList, listDir
 from nuitka.utils.ModuleNames import ModuleName
-from nuitka.utils.SharedLibraries import locateDLL
 from nuitka.utils.Utils import isMacOS, isWin32Windows
 
 # Use to detect the Qt plugin that is active and check for conflicts.
@@ -271,7 +263,9 @@ import %(binding_name)s.QtCore
         )
 
         if info is None:
-            self.sysexit("Error, it seems '%s' is not installed." % self.binding_name)
+            self.sysexit(
+                "Error, it seems '%s' is not installed or broken." % self.binding_name
+            )
 
         return info
 
@@ -291,7 +285,7 @@ import %(binding_name)s.QtCore
         """Get the path to the Qt webengine resources."""
         return os.path.join(self._getQtInformation().data_path, "resources")
 
-    def _getLibaryExecutablePath(self):
+    def _getLibraryExecutablePath(self):
         """Get the patch to Qt binaries."""
         return self._getQtInformation().library_executables_path
 
@@ -322,18 +316,21 @@ import %(binding_name)s.QtCore
 
     def _getQtBinDirs(self):
         for plugin_dir in self.getQtPluginDirs():
-            qt_bin_dir = os.path.normpath(os.path.join(plugin_dir, "..", "bin"))
+            if "PyQt" in self.binding_name:
+                qt_bin_dir = os.path.normpath(os.path.join(plugin_dir, "..", "bin"))
+
+                if os.path.isdir(qt_bin_dir):
+                    yield qt_bin_dir
+            else:
+                qt_bin_dir = os.path.normpath(os.path.join(plugin_dir, ".."))
 
-            if os.path.isdir(qt_bin_dir):
                 yield qt_bin_dir
 
     def hasPluginFamily(self, family):
-        for plugin_dir in self.getQtPluginDirs():
-            if os.path.isdir(os.path.join(plugin_dir, family)):
-                return True
-
-        # TODO: Special case "xml".
-        return False
+        return any(
+            os.path.isdir(os.path.join(plugin_dir, family))
+            for plugin_dir in self.getQtPluginDirs()
+        )
 
     def _getQmlDirectory(self):
         for plugin_dir in self.getQtPluginDirs():
@@ -385,7 +382,16 @@ import %(binding_name)s.QtCore
                 if not self.hasQtPluginSelected(qt_plugin_name):
                     continue
 
-                yield makeDllEntryPoint(
+                # TODO: The qpdf plugin is causing issues, and we would have to check
+                # here, already, if dependencies of the plugin will be available, and
+                # if not, then skip it, but that is for a future release, as it is
+                # blocking on macOS now.
+                if "qpdf" in os.path.basename(filename_relative) and not isExperimental(
+                    "qt-force-qpdf"
+                ):
+                    continue
+
+                yield self.makeDllEntryPoint(
                     source_path=filename,
                     dest_path=os.path.join(
                         self.binding_name,
@@ -400,7 +406,7 @@ import %(binding_name)s.QtCore
             return ModuleName(self.binding_name).getChildNamed(child_name)
 
     def getImplicitImports(self, module):
-        # Way too many indeed, pylint: disable=too-many-branches
+        # Way too many indeed, pylint: disable=too-many-branches,too-many-statements
 
         full_name = module.getFullName()
         top_level_package_name, child_name = full_name.splitPackageName()
@@ -521,6 +527,7 @@ import %(binding_name)s.QtCore
 
         if child_name in ("QtQuick", "QtQuickWidgets"):
             yield self._getChildNamed("QtQml")
+            yield self._getChildNamed("QtOpenGL")
 
         if child_name in ("QtQuickWidgets", "QtQml", "QtQuickControls2"):
             yield self._getChildNamed("QtQuick")
@@ -612,7 +619,7 @@ system Qt plugins, which may be from another Qt version.""",
             Code to insert and descriptive text (tuple), or (None, None).
         """
 
-        # This isonly relevant on standalone mode for Windows
+        # This is only relevant on standalone mode for Windows
         if not isStandaloneMode():
             return
 
@@ -626,7 +633,7 @@ if not path.startswith(__nuitka_binary_dir):
 """
             yield (
                 code,
-                "Adding binary folder to runtime 'PATH' environment variable for proper loading.",
+                "Adding binary folder to runtime 'PATH' environment variable for proper Qt loading.",
             )
 
     def considerDataFiles(self, module):
@@ -643,20 +650,21 @@ if not path.startswith(__nuitka_binary_dir):
             for filename in self._getQmlFileList(dlls=False):
                 filename_relative = os.path.relpath(filename, qml_plugin_dir)
 
-                yield makeIncludedDataFile(
+                yield self.makeIncludedDataFile(
                     source_path=filename,
                     dest_path=os.path.join(
                         qml_target_dir,
                         filename_relative,
                     ),
                     reason="Qt QML datafile",
+                    tags="qml",
                 )
         elif self.isQtWebEngineModule(full_name) and not self.webengine_done_data:
             self.webengine_done_data = True
 
             # TODO: This is probably wrong/not needed on macOS
             if not isMacOS():
-                yield makeIncludedGeneratedDataFile(
+                yield self.makeIncludedGeneratedDataFile(
                     data="""\
 [Paths]
 Prefix = .
@@ -668,7 +676,7 @@ Prefix = .
             resources_dir = self._getResourcesPath()
 
             for filename, filename_relative in listDir(resources_dir):
-                yield makeIncludedDataFile(
+                yield self.makeIncludedDataFile(
                     source_path=filename,
                     dest_path=os.path.join(
                         self._getResourcesTargetDir(), filename_relative
@@ -683,10 +691,11 @@ Prefix = .
                     filename_relative = os.path.relpath(filename, translations_path)
                     dest_path = self._getTranslationsTargetDir()
 
-                    yield makeIncludedDataFile(
+                    yield self.makeIncludedDataFile(
                         source_path=filename,
                         dest_path=os.path.join(dest_path, filename_relative),
                         reason="Qt translation",
+                        tags="translation",
                     )
 
     def getExtraDlls(self, module):
@@ -724,17 +733,20 @@ Prefix = .
                     [],
                 )
 
-                self.info("Including OpenSSL DLLs.")
-
+                count = 0
                 for filename in qt_bin_files:
                     basename = os.path.basename(filename).lower()
                     if basename in ("libeay32.dll", "ssleay32.dll"):
-                        yield makeDllEntryPoint(
+                        yield self.makeDllEntryPoint(
                             source_path=filename,
                             dest_path=basename,
                             package_name=full_name,
                         )
 
+                        count += 1
+
+                self.reportFileCount(full_name, count, section="OpenSSL")
+
             if (
                 "qml" in self.getQtPluginsSelected()
                 or "all" in self.getQtPluginsSelected()
@@ -745,7 +757,7 @@ Prefix = .
                 for filename in self._getQmlFileList(dlls=True):
                     filename_relative = os.path.relpath(filename, qml_plugin_dir)
 
-                    yield makeDllEntryPoint(
+                    yield self.makeDllEntryPoint(
                         source_path=filename,
                         dest_path=os.path.join(
                             qml_target_dir,
@@ -759,33 +771,34 @@ Prefix = .
                 if isWin32Windows():
                     opengl_dlls = ("libegl.dll", "libglesv2.dll", "opengl32sw.dll")
 
-                    self.info("Including OpenGL DLLs.")
-
+                    count = 0
                     for filename in qt_bin_files:
                         basename = os.path.basename(filename).lower()
 
                         if basename in opengl_dlls or basename.startswith(
                             "d3dcompiler_"
                         ):
-                            yield makeDllEntryPoint(
+                            yield self.makeDllEntryPoint(
                                 source_path=filename,
                                 dest_path=basename,
                                 package_name=full_name,
                             )
 
+                    self.reportFileCount(full_name, count, section="OpenGL")
+
         elif full_name == self.binding_name + ".QtNetwork":
             if not isWin32Windows():
-                dll_path = locateDLL("crypto")
+                dll_path = self.locateDLL("crypto")
                 if dll_path is not None:
-                    yield makeDllEntryPoint(
+                    yield self.makeDllEntryPoint(
                         source_path=dll_path,
                         dest_path=os.path.basename(dll_path),
                         package_name=full_name,
                     )
 
-                dll_path = locateDLL("ssl")
+                dll_path = self.locateDLL("ssl")
                 if dll_path is not None:
-                    yield makeDllEntryPoint(
+                    yield self.makeDllEntryPoint(
                         source_path=dll_path,
                         dest_path=os.path.basename(dll_path),
                         package_name=full_name,
@@ -794,7 +807,7 @@ Prefix = .
             self.webengine_done_binaries = True  # prevent multiple copies
             self.info("Including QtWebEngine executable.")
 
-            qt_web_engine_dir = self._getLibaryExecutablePath()
+            qt_web_engine_dir = self._getLibraryExecutablePath()
 
             for filename, filename_relative in listDir(qt_web_engine_dir):
                 if filename_relative.startswith("QtWebEngineProcess"):
@@ -841,7 +854,7 @@ Prefix = .
                         if os.path.basename(sub_dll_filename).startswith(badword):
                             yield sub_dll_filename
 
-    def onModuleEncounter(self, module_filename, module_name, module_kind):
+    def onModuleEncounter(self, module_name, module_filename, module_kind):
         top_package_name = module_name.getTopLevelPackageName()
 
         if isStandaloneMode():
@@ -854,7 +867,7 @@ Prefix = .
                         """\
 Unwanted import of '%(unwanted)s' that conflicts with '%(binding_name)s' encountered, preventing
 its use. As a result an "ImportError" might be given at run time. Uninstall it for full compatible
-behaviour with the uncompiled code to debug it."""
+behavior with the uncompiled code to debug it."""
                         % {
                             "unwanted": top_package_name,
                             "binding_name": self.binding_name,
@@ -960,28 +973,27 @@ class NuitkaPluginPySide2Plugins(NuitkaPluginQtBindingsPluginBase):
         if self._getNuitkaPatchLevel() < 1:
             self.warning(
                 """\
-This PySide2 version only partially supported through workarounds, full support: https://nuitka.net/pages/pyside2.html"""
+For the standard PySide2 incomplete workarounds are applied. For full support: https://nuitka.net/pages/pyside2.html"""
             )
 
             if python_version < 0x360:
                 self.sysexit(
-                    "Error, unpatched PySide2 is not supported before CPython <3.6."
+                    """\
+The standard PySide2 is not supported before CPython <3.6. For full support: https://nuitka.net/pages/pyside2.html"""
                 )
 
         NuitkaPluginQtBindingsPluginBase.__init__(
             self, qt_plugins=qt_plugins, no_qt_translations=no_qt_translations
         )
 
-    def onModuleEncounter(self, module_filename, module_name, module_kind):
-        # Enforce recursion in to multiprocessing for accelerated mode, which
-        # would normally avoid this.
+    def onModuleEncounter(self, module_name, module_filename, module_kind):
         if module_name == self.binding_name and self._getNuitkaPatchLevel() < 1:
             return True, "Need to monkey patch PySide2 for abstract methods."
 
         return NuitkaPluginQtBindingsPluginBase.onModuleEncounter(
             self,
-            module_filename=module_filename,
             module_name=module_name,
+            module_filename=module_filename,
             module_kind=module_kind,
         )
 
@@ -1083,6 +1095,10 @@ PySide2.QtCore.QObject.__init_subclass__ = my_init_subclass
 Monkey patching classes derived from PySide2 base classes to pass PySide2 checks.""",
             )
 
+    def getModuleSpecificDllPaths(self, module_name):
+        if module_name.hasNamespace("PySide2"):
+            yield self.locateModule("shiboken2")
+
 
 class NuitkaPluginDetectorPySide2Plugins(NuitkaPluginBase):
     detector_for = NuitkaPluginPySide2Plugins
@@ -1115,6 +1131,10 @@ class NuitkaPluginPySide6Plugins(NuitkaPluginQtBindingsPluginBase):
 Only PySide 6.1.2 or higher (or dev branch compiled), otherwise callbacks won't work."""
             )
 
+    def getModuleSpecificDllPaths(self, module_name):
+        if module_name.hasNamespace("PySide6"):
+            yield self.locateModule("shiboken6")
+
 
 class NuitkaPluginDetectorPySide6Plugins(NuitkaPluginBase):
     detector_for = NuitkaPluginPySide6Plugins
