@@ -33,6 +33,7 @@ from nuitka.PythonVersions import python_version
 from nuitka.tools.testing.Common import (
     addToPythonPath,
     executeAfterTimePassed,
+    getDebugPython,
     getTempDir,
     getTestingCPythonOutputsCacheDir,
     killProcess,
@@ -226,8 +227,9 @@ def main():
     ignore_warnings = hasArg("ignore_warnings")
     expect_success = hasArg("expect_success")
     expect_failure = hasArg("expect_failure")
-    python_debug = hasArg("python_debug")
+    python_debug = hasArg("python_debug") or hasArg("--python-debug")
     module_mode = hasArg("--module")
+    module_entry_point = hasArgValue("--module-entry-point")
     coverage_mode = hasArg("coverage")
     two_step_execution = hasArg("two_step_execution")
     binary_python_path = hasArg("binary_python_path")
@@ -325,22 +327,13 @@ def main():
 
     extra_options = os.environ.get("NUITKA_EXTRA_OPTIONS", "").split()
 
-    if "--python-debug" in extra_options or "--python-dbg" in extra_options:
+    if os.path.normcase(os.environ["PYTHON"]).endswith(("-dbg", "-debug", "_d.exe")):
+        python_debug = True
+    elif "--python-debug" in extra_options or "--python-dbg" in extra_options:
         python_debug = True
 
     if python_debug:
-        if os.path.exists(os.path.join("/usr/bin/", os.environ["PYTHON"] + "-dbg")):
-            os.environ["PYTHON"] += "-dbg"
-
-        if os.name == "nt":
-            if os.path.exists(os.environ["PYTHON"][:-4] + "_d.exe"):
-                os.environ["PYTHON"] = os.environ["PYTHON"][:-4] + "_d.exe"
-
-    if os.environ["PYTHON"].endswith("-dbg"):
-        python_debug = True
-
-    if os.environ["PYTHON"].lower().endswith("_d.exe"):
-        python_debug = True
+        os.environ["PYTHON"] = getDebugPython() or os.environ["PYTHON"]
 
     if comparison_mode:
         my_print(
@@ -375,11 +368,15 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
         if module_name.endswith(".py"):
             module_name = module_name[:-3]
 
+        mini_script = "import %s" % module_name
+        if module_entry_point:
+            mini_script += "; %s.%s()" % (module_name, module_entry_point)
+
         cpython_cmd = [
             os.environ["PYTHON"],
             "-c",
-            "import sys; sys.path.append(%s); import %s"
-            % (repr(os.path.dirname(filename)), module_name),
+            "import sys; sys.path.append(%s); %s"
+            % (repr(os.path.dirname(filename)), mini_script),
         ]
 
         if no_warnings:
@@ -409,7 +406,7 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
         # Would need to extract which "python" this is going to use.
         assert not coverage_mode, "Not implemented for binaries."
 
-        nuitka_call = [os.environ["NUITKA"]]
+        nuitka_call = [os.environ["PYTHON"], os.environ["NUITKA"]]
     else:
         if comparison_mode:
             nuitka_call = [
@@ -476,7 +473,7 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
     if binary_python_path:
         addToPythonPath(os.path.dirname(os.path.abspath(filename)))
 
-    if keep_python_path or binary_python_path:
+    if (keep_python_path or binary_python_path) and not coverage_mode:
         extra_options.append("--execute-with-pythonpath")
 
     if report:
@@ -566,13 +563,7 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
         if module_name.endswith(".py"):
             module_name = module_name[:-3]
 
-        nuitka_cmd2 = [
-            os.environ["PYTHON"],
-            "-W",
-            "ignore",
-            "-c",
-            "import %s" % module_name,
-        ]
+        nuitka_cmd2 = [os.environ["PYTHON"], "-W", "ignore", "-c", mini_script]
     else:
         exe_filename = os.path.basename(filename)
 
